// Initialize side panel behavior
// let BASE_URL = "http://localhost:8000";
let BASE_URL = "https://www.onboardassistant.com";

chrome.sidePanel
    .setPanelBehavior({ openPanelOnActionClick: true })
    .catch((error) => console.error('Error setting panel behavior:', error));

// Handle extension icon click
chrome.action.onClicked.addListener(async (tab) => {
    try {
        console.log("Extension icon clicked");
        await chrome.sidePanel.setOptions({
            tabId: tab.id,
            path: 'sidebar/sidebar.html',
            enabled: true
        });

        await chrome.sidePanel.open({
            tabId: tab.id
        });
    } catch (error) {
        console.error("Error opening side panel:", error);
    }
});

// Message handling
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    console.log("Received message:", request.type);

    switch (request.type) {
        case 'userChat':
            handleChat(request);
            break;
        case 'locateRequest':
            LocateRequest(request);
            break;
        default:
            console.warn("Unknown message type:", request.type);
    }
});

async function getCoordinates(screenshot, text) {
  // const url = `${BASE_URL}/api/get_coordinates/`;
  
  // const response = await fetch(url, {
  //   method: 'POST',
  //   headers: { 'Content-Type': 'application/json' },
  //   body: JSON.stringify({ screenshot, text }) 
  // });
  // if (!response.ok) {
  //   throw new Error(`API request failed: ${response.status}`);
  // }
  // return await response.json();

  return { x: 190, y: 120 };
}

async function LocateRequest(request) {
  if (!request) {
    console.error("Invalid request");
    return;
  }

  const stepIndex = request.stepIndex;
  const rationale = request.rationale;
  const box = request.bbox;

  chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
    if (!tabs || tabs.length === 0) {
      console.error("No active tab found");
      return;
    }

    const activeTab = tabs[0];

    let data;

    chrome.tabs.captureVisibleTab(activeTab.windowId, { format: 'png' }, async (screenshotUrl) => {
      if (chrome.runtime.lastError) {
        console.error('Screenshot capture failed:', chrome.runtime.lastError);
        return;
      }

      try {
        const response = await fetch(screenshotUrl);
        const screenshotBlob = await response.blob();
        data = await getCoordinates(screenshotBlob, rationale);

        console.log('Coordinates from server:', data);

      } catch (err) {
        console.error('Error in screenshot/coordinates:', err);
      }

      const box = {
        x: data.x,
        y: data.y
      }

      chrome.scripting.executeScript(
        {
          target: { tabId: activeTab.id },
          files: ["content.js"]
        },
        () => {
          if (chrome.runtime.lastError) {
            console.error("Failed to inject content script:", chrome.runtime.lastError.message);
            return;
          }
          console.log("Content script injected successfully");

          chrome.tabs.sendMessage(
            activeTab.id,
            { type: 'locateRequest', bbox: box },
            (response) => {
              if (chrome.runtime.lastError) {
                console.error("Error sending message to content script:", chrome.runtime.lastError.message);
              } else if (response && response.success === true) {
                console.log("Message successfully relayed to content script", response);
              } else {
                console.warn("No response or unexpected response from content script");
              }
            }
          );
        }
      );
    });
  });
}


// Chat handling
function handleChat(request) {
    const userMessage = request.message;
    const authToken = request.authToken;

    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        const activeTab = tabs[0];

        if (!activeTab) {
            console.error('No active tab found.');
            sendErrorResponse('No active tab found. Please try again.');
            return;
        }

        chrome.tabs.captureVisibleTab(activeTab.windowId, { format: 'png' }, async (screenshotUrl) => {
            if (chrome.runtime.lastError) {
                console.error('Screenshot capture failed:', chrome.runtime.lastError);
                sendErrorResponse('Failed to capture screenshot. Please try again.');
                return;
            }

            try {
                const url = activeTab.url;
                const tabTitle = activeTab.title;

                // Convert data URL to blob
                const response = await fetch(screenshotUrl);
                const screenshotBlob = await response.blob();
                
                const prompt = `${userMessage}`;
                
                const agentResponse = await callAgentAPI(prompt, screenshotBlob, authToken, url);
                
                chrome.runtime.sendMessage({
                    type: 'agentResponse',
                    message: agentResponse
                });
            } catch (error) {
                console.error('Error processing chat:', error);
                sendErrorResponse('Failed to get response from agent.');
            }
        });
    });
}


async function callAgentAPI(prompt, screenshotBlob, authToken, url) {
  try {
      const base64screenshot = await blobToBase64(screenshotBlob);

      const requestBody = {
          text_msg: prompt,
          image_data: base64screenshot,
          url: url,
          version: 'v1'
      };

      const result = await chrome.storage.local.get(['sessionId']);
      if (result.sessionId) {
          console.log('Using existing session ID:', result.sessionId);
          requestBody.session_id = result.sessionId;
      }

      let response = await fetch(`${BASE_URL}/api/accounts/generate_content`, {
          method: 'POST',
          credentials: 'include',
          headers: {
              'Content-Type': 'application/json',
          },
          body: JSON.stringify(requestBody)
      });
    
      if (response.status === 401 || response.status === 403) {
          // Await the authentication request
          const authResponse = await fetch(`${BASE_URL}/api/accounts/auth/customer`, {
              method: 'POST',
              credentials: 'include',
              headers: {
                  'Content-Type': 'application/json',
              },
              body: JSON.stringify({
                  token: authToken,
                  env: 'extension'
              })
          });
          

          response = await fetch(`${BASE_URL}/api/accounts/generate_content`, {
              method: 'POST',
              credentials: 'include',
              headers: {
                  'Content-Type': 'application/json'
              },
              body: JSON.stringify(requestBody)
          });
      }

      if (!response.ok) {
          throw new Error(`Agent request failed: ${response.statusText}`);
      }

      const output = await response.json();
      if ('session_id' in output) {
        await chrome.storage.local.set({ sessionId: output["session_id"] });
      } else {
        await chrome.storage.local.set({ sessionId: null });
      }

      return output["content"];
  } catch (error) {
      console.error('API call error:', error);
      throw error;
  }
}

// Utility functions
function blobToBase64(blob) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onloadend = () => {
            const base64String = reader.result.split(',')[1];
            resolve(base64String);
        };
        reader.onerror = reject;
        reader.readAsDataURL(blob);
    });
}

function sendErrorResponse(message) {
    chrome.runtime.sendMessage({
        type: 'error',
        message: `Error: ${message}`
    });
}

// Error handling
chrome.runtime.onInstalled.addListener(() => {
    console.log('Extension installed');
});

chrome.runtime.onSuspend.addListener(() => {
    console.log('Extension being suspended');
});
